@@ -119,6 +119,8 @@ package
     rgnop; *AUTOLOAD = \&Prima::Const::AUTOLOAD;# Region operations
 package
     rgn; *AUTOLOAD = \&Prima::Const::AUTOLOAD;# Region.rect_inside() results
+package
+    fm; *AUTOLOAD = \&Prima::Const::AUTOLOAD;# fill modes
 
 1;
 
@@ -402,6 +404,14 @@ See L<Prima::File>
 	fe::Write
 	fe::Exception
 
+=head2 fm::  - fill modes 
+
+See L<Prima::Drawable/fillMode>
+
+	fp::Alternate
+	fp::Winding
+	fp::Overlay
+
 =head2 fp::  - standard fill pattern indices
 
 See L<Prima::Drawable/fillPattern>
@@ -952,6 +962,7 @@ See also L<Prima::Application/get_system_value>
 	                       0 if it doesn't, -1 if unknown
 	sv::LayeredWidgets   - 1 if system supports layering
 	sv::DWM              - 1 if system supports DWM API
+	sv::FixedPointerSize - 0 if system doesn't support arbitrary sized pointers and will resize custom icons to the system size
 
 =head2 ta::  - alignment constants
 
