@@ -10,6 +10,11 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
+#define Box _prima_Box
+#include <X11/Xregion.h>
+#undef Box
+#define Box BoxRec
+#undef Box
 #include <X11/Xresource.h>
 #include <X11/cursorfont.h>
 #ifdef HAVE_X11_EXTENSIONS_XRENDER_H
@@ -300,7 +305,7 @@ typedef struct _timer_sys_data
 typedef struct
 {
 	Region region;
-	int height;
+	int aperture;
 } RegionSysData, *PRegionSysData;
 
 struct  _drawable_sys_data;
@@ -734,6 +739,7 @@ typedef struct _drawable_sys_data
 	XRectangle clip_rect;
 	FillPattern fill_pattern, saved_fill_pattern;
 	Point fill_pattern_offset, saved_fill_pattern_offset;
+	int fill_mode, saved_fill_mode;
 	Pixmap fp_pixmap;
 #if defined(sgi) && !defined(__GNUC__)
 /* multiple compilation and runtime errors otherwise. must be some alignment tricks */
@@ -742,6 +748,7 @@ typedef struct _drawable_sys_data
 	int rop, paint_rop;
 	int rop2, paint_rop2;
 	int line_style, line_width;
+	float miter_limit;
 	unsigned char *dashes, *paint_dashes;
 	int ndashes, paint_ndashes;
 	Point clip_mask_extent, shape_extent, shape_offset;
@@ -1375,3 +1382,6 @@ prima_find_toplevel_window(Handle self);
 
 extern Byte*
 prima_mirror_bits( void);
+
+extern int
+prima_copy_region_data(void * region);
