@@ -1256,6 +1256,7 @@ register_constants( void)
 	register_dbt_constants();
 	register_rgn_constants();
 	register_rgnop_constants();
+	register_fm_constants();
 }
 
 XS( Object_alive_FROMPERL);
@@ -1887,8 +1888,11 @@ prima_omp_set_num_threads(int num)
 SV *
 prima_array_new( size_t size)
 {
-	SV * sv = newSV( size );
+	SV * sv;
+	if ( size == 0 ) return newSVpv("", 0);
+	sv = newSV( size );
 	SvPOK_only(sv);
+	SvCUR_set(sv, size );
 	return sv;
 }
 
@@ -1984,6 +1988,76 @@ prima_read_point( SV *rv_av, int * pt, int number, char * error)
 	return result;
 }
 
+void *
+prima_read_array( SV * points, char * procName, Bool integer, int div, int min, int max, int * n_points )
+{
+	AV * av;
+	int i, count, psize;
+	void * p;
+
+	psize = integer ? sizeof(int) : sizeof(double);
+	if ( !SvROK( points) || ( SvTYPE( SvRV( points)) != SVt_PVAV)) {
+		warn("Invalid array reference passed to %s", procName);
+		return NULL;
+	}
+	av = ( AV *) SvRV( points);
+	count = av_len( av) + 1;
+	if ( min == max && count != min * div ) {
+		warn("%s: array must contain %d elements", procName, min * div);
+		return NULL;
+	}
+	if ( count < min * div ) {
+		warn("%s: array must contain at least %d elements", procName, min * div);
+		return NULL;
+	}
+	if ( max >= 0 && count > max * div ) {
+		warn("%s: array must contain maximum %d elements", procName, max * div);
+		return NULL;
+	}
+	if ( count % div != 0 ) {
+		warn("%s: number of elements in an array must be a multiple of %d", procName, div);
+		return NULL;
+	}
+	if ( n_points)
+		*n_points = count / div;
+	if ( count == 0)
+		return NULL;
+
+	{
+		void * ref;
+		char * pack, req = integer ? 'i' : 'd';
+		if ( prima_array_parse( points, &ref, NULL, &pack ) && *pack == req) {
+			if (!( p = malloc( psize * count))) {
+				warn("not enough memory");
+				return false;
+			}
+			memcpy( p, ref, psize * count);
+			return p;
+		}
+	}
+
+
+	if (!( p = malloc( psize * count))) {
+		warn("not enough memory");
+		return NULL;
+	}
+
+	for ( i = 0; i < count; i++)
+	{
+		SV** psv = av_fetch( av, i, 0);
+		if ( psv == NULL) {
+			free( p);
+			warn("Array panic on item %d on %s", i, procName);
+			return NULL;
+		}
+		if ( integer )
+			*(((int*)p) + i) = SvIV( *psv);
+		else
+			*(((double*)p) + i) = SvNV( *psv);
+	}
+
+	return p;
+}
 
 #ifdef __cplusplus
 }
