@@ -254,8 +254,12 @@ apc_pointer_set_user( Handle self, Handle icon, Point hotSpot)
 	apcErrClear;
 	direct = direct_pointer_change( self);
 	hotSpot. y = guts. pointerSize. y - hotSpot. y - 1;
-	cursor = icon ? image_make_icon_handle( icon, guts. pointerSize, &hotSpot) : nil;
-	if ( apcError) return false;
+	if (icon) {
+		Point sz = { PImage(icon)->w, PImage(icon)-> h };
+		cursor = image_make_icon_handle( icon, sz, &hotSpot);
+		if ( apcError) return false;
+	} else
+		cursor = nil;
 
 	if ( sys pointer2) {
 		if ( direct) SetCursor( NULL);
