@@ -480,7 +480,7 @@ apc_gp_fill_chord( Handle self, int x, int y, int dX, int dY, double angleStart,
 	int compl, needf;
 
 	compl = arc_completion( &angleStart, &angleEnd, &needf);
-	comp = stylus_complex( &sys stylus, ps);
+	comp = ((sys psFillMode & fmOverlay) == 0) || stylus_complex( &sys stylus, ps);
 	y = sys lastSize. y - y - 1;
 	STYLUS_USE_BRUSH( ps);
 
@@ -510,7 +510,7 @@ apc_gp_fill_ellipse( Handle self, int x, int y, int dX, int dY)
 	Bool ok = true;
 	HDC     ps  = sys ps;
 	HGDIOBJ old;
-	Bool    comp = stylus_complex( &sys stylus, ps);
+	Bool    comp = ((sys psFillMode & fmOverlay) == 0) || stylus_complex( &sys stylus, ps);
 	STYLUS_USE_BRUSH( ps);
 	y = sys lastSize. y - y - 1;
 	if ( comp) {
@@ -553,7 +553,12 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 
 	for ( i = 0; i < numPts; i++) points[ i]. y = dy - points[ i]. y - 1;
 
-	if ( !stylus_complex( &sys stylus, ps)) {
+	if (( sys psFillMode & fmOverlay) == 0) {
+		HGDIOBJ old = SelectObject( ps, hPenHollow);
+		STYLUS_USE_BRUSH( ps);
+		if ( !( ok = Polygon( ps, ( POINT *) points, numPts))) apiErr;
+		SelectObject( ps, old);
+	} else if ( !stylus_complex( &sys stylus, ps)) {
 		HPEN old = SelectObject( ps, CreatePen( PS_SOLID, 1, sys stylus. brush. lb. lbColor));
 		STYLUS_USE_BRUSH( ps);
 		if ( !( ok = Polygon( ps, ( POINT *) points, numPts))) apiErr;
@@ -641,7 +646,7 @@ apc_gp_fill_sector( Handle self, int x, int y, int dX, int dY, double angleStart
 	int compl, needf;
 
 	compl = arc_completion( &angleStart, &angleEnd, &needf);
-	comp = stylus_complex( &sys stylus, ps);
+	comp = ((sys psFillMode & fmOverlay) == 0) || stylus_complex( &sys stylus, ps);
 
 	pts[ 0]. x = x + cos( angleEnd / GRAD) * dX / 2 + 0.5;
 	pts[ 0]. y = newY - sin( angleEnd / GRAD) * dY / 2 + 0.5;
@@ -1262,12 +1267,11 @@ apc_gp_get_color( Handle self)
 	return remap_color( sys ps ? sys stylus. pen. lopnColor : sys lbs[0], false);
 }
 
-Bool
-apc_gp_get_fill_winding( Handle self)
+int
+apc_gp_get_fill_mode( Handle self)
 {
 	objCheck 0;
-	if ( ! sys ps) return sys fillWinding;
-	return GetPolyFillMode( sys ps) == WINDING;
+	return sys ps ? sys psFillMode : sys fillMode;
 }
 
 static Handle ctx_le2PS_ENDCAP[] = {
@@ -1348,6 +1352,16 @@ apc_gp_get_line_pattern( Handle self, unsigned char * buffer)
 	}
 }
 
+float
+apc_gp_get_miter_limit( Handle self)
+{
+	FLOAT ml;
+	objCheck 0;
+	if ( !sys ps) return sys miterLimit;
+	if (! GetMiterLimit( sys ps, &ml)) return 0;
+	return (float)ml;
+}
+
 Color
 apc_gp_get_nearest_color( Handle self, Color color)
 {
@@ -1708,13 +1722,14 @@ apc_gp_set_color( Handle self, Color color)
 }
 
 Bool
-apc_gp_set_fill_winding( Handle self, Bool fillWinding)
+apc_gp_set_fill_mode( Handle self, int fillMode)
 {
 	objCheck false;
-	if ( sys ps)
-		SetPolyFillMode( sys ps, fillWinding ? WINDING : ALTERNATE);
-	else
-		sys fillWinding = fillWinding;
+	if ( sys ps) {
+		SetPolyFillMode( sys ps, ((fillMode & fmWinding) == fmAlternate) ? ALTERNATE : WINDING);
+		sys psFillMode = fillMode;
+	} else
+		sys fillMode = fillMode;
 	return true;
 }
 
@@ -1879,6 +1894,18 @@ apc_gp_set_line_pattern( Handle self, unsigned char * pattern, int len)
 	return true;
 }
 
+Bool
+apc_gp_set_miter_limit( Handle self, float miter_limit)
+{
+	objCheck false;
+	if ( !sys ps) {
+		sys miterLimit = miter_limit;
+		return true;
+	} else 
+		return SetMiterLimit( sys ps, (FLOAT) miter_limit, NULL);
+}
+
+
 Bool
 apc_gp_set_palette( Handle self)
 {
