@@ -18,7 +18,7 @@ extern "C" {
 
 #define GET_REGION(obj) (&(dsys(obj)s.region))
 #define REGION GET_REGION(self)->region
-#define HEIGHT GET_REGION(self)->height
+#define APERTURE GET_REGION(self)->aperture
 
 HRGN
 region_create( Handle mask)
@@ -118,25 +118,73 @@ static Bool
 rgn_empty(Handle self)
 {
 	REGION = CreateRectRgn(0,0,0,0);
-	HEIGHT = 0;
+	APERTURE = 0;
 	return true;
 }
 
 static Bool
-rgn_rect(Handle self, Box r)
+rgn_rect(Handle self, int count, Box * r)
 {
-	REGION = CreateRectRgn(r.x, 0, r.width+r.x, r.height);
-	HEIGHT = r.y + r.height;
+	int i, h;
+	Box * rr;
+	h = r->y + r->height;
+	for ( i = 0, rr = r; i < count; i++, rr++) {
+		if ( h < rr->y + rr->height)
+			h = rr->y + rr->height;
+	}
+	REGION = CreateRectRgn(r->x, h - r->y - r->height, r->width+r->x, h - r->y);
+	APERTURE = h;
+	for ( i = 1, r++; i < count; i++, r++) {
+		HRGN reg = CreateRectRgn(r->x, h - r->y - r->height, r->width+r->x, h - r->y);
+		CombineRgn( REGION, REGION, reg, RGN_OR);
+		DeleteObject(reg);
+	}
 	return true;
 }
 
+static void dump( HRGN r )
+{
+	int i, size;
+	PRegionRec ret;
+	RECT *src;
+	RGNDATA *rgndata;
+
+	size = GetRegionData( r, 0, NULL);
+	if ( !( rgndata = malloc(size))) {
+		warn("Not enough memory\n");
+		return ;
+	}
+	size = GetRegionData( r, size, rgndata);
+	if ( size == 0) return ;
+
+	ret = malloc(sizeof(RegionRec) + sizeof(Box) * rgndata-> rdh. nCount );
+	if ( ret == NULL ) {
+		free(ret);
+		warn("Not enough memory\n");
+		return ;
+	}
+
+	src = (RECT*) &(rgndata->Buffer);
+	ret-> data. box. n_boxes = rgndata->rdh. nCount;
+	for ( i = 0; i < ret->data. box. n_boxes; i++, src++) {
+		printf("%d: [%ld %ld %ld %ld]\n", i, src->left, src->top, src->right, src->bottom); 
+	}
+	free(rgndata);
+	free(ret);
+}
+
 static Bool
 rgn_polygon(Handle self, PolygonRegionRec * r)
 {
-	int i, max;
+	int i, max, open, xp_points;
 	POINT * xp;
 
-	if ( !( xp = malloc( sizeof(POINT) * r->n_points ))) {
+	open =
+		r->points[r->n_points-1].x != r->points[0].x ||
+		r->points[r->n_points-1].y != r->points[0].y;
+	xp_points = r->n_points + (open ? 1 : 0);
+
+	if ( !( xp = malloc( sizeof(POINT) * xp_points ))) {
 		warn("Not enough memory");
 		return false;
 	}
@@ -145,14 +193,119 @@ rgn_polygon(Handle self, PolygonRegionRec * r)
 		if ( max < r->points[i].y)
 			max = r->points[i].y;
 	}
+	max++;
 	for ( i = 0; i < r->n_points; i++) {
 		xp[i].x = r->points[i].x;
-		xp[i].y = max - r->points[i].y;
+		xp[i].y = max - r->points[i].y - 1;
+	}
+	if ( open ) {
+		xp[i].x = r->points[0].x;
+		xp[i].y = max - r->points[0].y - 1;
 	}
 
-	HEIGHT = max;
-	REGION = CreatePolygonRgn( xp, r->n_points, r-> winding ? WINDING : ALTERNATE );
+	APERTURE = max;
+	REGION = CreatePolygonRgn( xp, r->n_points, 
+		((r-> fill_mode & fmWinding) == fmAlternate) ? ALTERNATE : WINDING);
+	if (( r->fill_mode & fmOverlay) == 0) goto NO_OVERLAY;
 
+	/* superimpose polyline points using Bresenham
+	because windows regions are as broken as filled shapes */
+	for ( i = 0; i < xp_points-1; i++) {
+		int curr_maj, curr_min, to_maj, delta_maj, delta_min;
+		int delta_y, delta_x;
+		int dir = 0, d, d_inc1, d_inc2;
+		int inc_maj, inc_min;
+		int x, y, acc_x = 0, acc_y = INT_MIN, ox;
+		POINT a = xp[i], b = xp[i+1];
+		delta_y = b.y - a.y;
+		delta_x = b.x - a.x;
+		if (abs(delta_y) > abs(delta_x)) dir = 1;
+
+		if (dir) {
+			curr_maj = a.y;
+			curr_min = a.x;
+			to_maj = b.y;
+			delta_maj = delta_y;
+			delta_min = delta_x;
+		} else {
+			curr_maj = a.x;
+			curr_min = a.y;
+			to_maj = b.x;
+			delta_maj = delta_x;
+			delta_min = delta_y;
+		}
+
+		if (delta_maj != 0)
+			inc_maj = (abs(delta_maj)==delta_maj ? 1 : -1);
+		else
+			inc_maj = 0;
+
+		if (delta_min != 0)
+			inc_min = (abs(delta_min)==delta_min ? 1 : -1);
+		else
+			inc_min = 0;
+
+		delta_maj = abs(delta_maj);
+		delta_min = abs(delta_min);
+
+		d      = (delta_min << 1) - delta_maj;
+		d_inc1 = (delta_min << 1);
+		d_inc2 = ((delta_min - delta_maj) << 1);
+
+		while(1) {
+			ox = x;
+			if (dir) {
+				x = curr_min;
+				y = curr_maj;
+			} else {
+				x = curr_maj;
+				y = curr_min;
+			}
+			if ( acc_y != y ) {
+				if ( acc_y > INT_MIN) {
+					HRGN reg;
+					int x1, x2;
+					if (ox < acc_x) {
+						x1 = ox;
+						x2 = acc_x;
+					} else {
+						x1 = acc_x;
+						x2 = ox;
+					}
+					reg = CreateRectRgn(x1, acc_y, x2+1, acc_y + 1);
+					CombineRgn( REGION, REGION, reg, RGN_OR);
+					DeleteObject(reg);
+				}
+				acc_x = x;
+				acc_y = y;
+			}
+
+			if (curr_maj == to_maj) break;
+			curr_maj += inc_maj;
+			if (d < 0) {
+				d += d_inc1;
+			} else {
+				d += d_inc2;
+				curr_min += inc_min;
+			}
+		}
+		if ( acc_y > INT_MIN) {
+			HRGN reg;
+			int x1, x2;
+			if (x < acc_x) {
+				x1 = x;
+				x2 = acc_x;
+			} else {
+				x1 = acc_x;
+				x2 = x;
+			}
+			reg = CreateRectRgn(x1, acc_y, x2+1, acc_y + 1);
+			CombineRgn( REGION, REGION, reg, RGN_OR);
+			DeleteObject(reg);
+		}
+	}
+
+NO_OVERLAY:
 	free( xp );
 	return true;
 }
@@ -165,7 +318,7 @@ rgn_image(Handle self, Handle image)
 	if ( !REGION )
 		REGION = CreateRectRgn(0,0,0,0);
 	else
-		HEIGHT = PImage(image)->h;
+		APERTURE = PImage(image)->h;
 	return true;
 }
 
@@ -176,7 +329,7 @@ apc_region_create( Handle self, PRegionRec rec)
 	case rgnEmpty:
 		return rgn_empty(self);
 	case rgnRectangle:
-		return rgn_rect(self, rec->data.box);
+		return rgn_rect(self, rec->data.box.n_boxes, rec->data.box.boxes);
 	case rgnPolygon:
 		return rgn_polygon(self, &rec->data.polygon);
 	case rgnImage:
@@ -223,11 +376,11 @@ apc_region_combine( Handle self, Handle other_region, int rgnop)
 
 	if ( rgnop == rgnopCopy ) {
 		ok = CombineRgn( REGION, r2->region, NULL, RGN_COPY);
-		HEIGHT = r2-> height;
+		APERTURE = r2-> aperture;
 		return ok;
 	}
 
-	d = HEIGHT - r2-> height;
+	d = APERTURE - r2-> aperture;
 	if ( d > 0 )
 		OffsetRgn( r2-> region, 0, d);
 	else
@@ -239,7 +392,7 @@ apc_region_combine( Handle self, Handle other_region, int rgnop)
 	if ( d > 0 )
 		OffsetRgn( r2-> region, 0, -d);
 	else
-		HEIGHT = r2-> height;
+		APERTURE = r2-> aperture;
 
 	return ok;
 }
@@ -247,7 +400,7 @@ apc_region_combine( Handle self, Handle other_region, int rgnop)
 Bool
 apc_region_point_inside( Handle self, Point p)
 {
-	return PtInRegion( REGION, p.x, HEIGHT - p.y - 1);
+	return PtInRegion( REGION, p.x, APERTURE - p.y - 1);
 }
 
 int
@@ -258,9 +411,9 @@ apc_region_rect_inside( Handle self, Rect r)
 	int ret;
 
 	q. left   = r. left;
-	q. top    = HEIGHT - r. bottom;
+	q. top    = APERTURE - r. bottom;
 	q. right  = r. right + 1;
-	q. bottom = HEIGHT - r. top - 1;
+	q. bottom = APERTURE - r. top - 1;
 	if ( RectInRegion( REGION, &q) == 0 ) return rgnOutside;
 
 	t1 = CreateRectRgnIndirect(&q);
@@ -297,7 +450,7 @@ apc_region_get_box( Handle self)
 		return box;
 	}
 	box. x      = xr. left;
-	box. y      = HEIGHT - xr. bottom;
+	box. y      = APERTURE - xr. bottom;
 	box. width  = xr. right - xr. left;
 	box. height = xr. bottom - xr. top;
 	return box;
@@ -379,7 +532,7 @@ apc_gp_get_region( Handle self, Handle mask)
 		return false;
 	GetRgnBox(rgn, &rect);
 	OffsetRgn( rgn, sys transform2. x, sys transform2. y - rect.top);
-	GET_REGION(mask)-> height = sys lastSize. y - rect.top;
+	GET_REGION(mask)-> aperture = sys lastSize. y - rect.top;
 	return true;
 }
 
@@ -398,7 +551,7 @@ apc_gp_set_region( Handle self, Handle region)
 	rgn = CreateRectRgn(0,0,0,0);
 	CombineRgn(rgn, GET_REGION(region)->region, NULL, RGN_COPY);
 	OffsetRgn( rgn, -sys transform2. x, -sys transform2. y);
-	OffsetRgn( rgn, 0, sys lastSize.y - GET_REGION(region)->height);
+	OffsetRgn( rgn, 0, sys lastSize.y - GET_REGION(region)->aperture);
 	if ( is_apt(aptLayeredPaint) && sys layeredParentRegion )
 		CombineRgn( rgn, rgn, sys layeredParentRegion, RGN_AND);
 	SelectClipRgn( sys ps, rgn);
@@ -406,6 +559,46 @@ apc_gp_set_region( Handle self, Handle region)
 	return true;
 }
 
+PRegionRec
+apc_region_copy_rects( Handle self)
+{
+	int i, aperture, size;
+	PRegionRec ret;
+	Box *dst;
+	RECT *src;
+	RGNDATA *rgndata;
+
+	size = GetRegionData( REGION, 0, NULL);
+	if ( !( rgndata = malloc(size))) {
+		warn("Not enough memory\n");
+		return NULL;
+	}
+	size = GetRegionData( REGION, size, rgndata);
+	if ( size == 0) return NULL;
+
+	ret = malloc(sizeof(RegionRec) + sizeof(Box) * rgndata-> rdh. nCount );
+	if ( ret == NULL ) {
+		free(ret);
+		warn("Not enough memory\n");
+		return NULL;
+	}
+
+	ret-> type = rgnRectangle;
+	ret-> data. box. n_boxes = rgndata->rdh. nCount;
+	src = (RECT*) &(rgndata->Buffer);
+	dst = ret-> data. box. boxes = (Box*) (((Byte*)ret) + sizeof(RegionRec));
+	aperture = APERTURE;
+	for ( i = 0; i < ret->data. box. n_boxes; i++, src++, dst++) {
+		dst-> x = src-> left;
+		dst-> y = aperture - src-> bottom;
+		dst-> width  = src-> right - src->left;
+		dst-> height = src-> bottom - src->top;
+		/* printf("%d: %ld %ld %ld %ld => %d %d %d %d\n", aperture, src->left, src->top, src->right, src->bottom, dst->x, dst->y, dst-> width, dst->height); */
+	}
+	free(rgndata);
+
+	return ret;
+}
 
 #ifdef __cplusplus
 }
