@@ -1508,7 +1508,7 @@ hwnd_enter_paint( Handle self)
 
 	apc_gp_set_text_opaque( self, is_apt( aptTextOpaque));
 	apc_gp_set_text_out_baseline( self, is_apt( aptTextOutBaseline));
-	apc_gp_set_fill_winding( self, sys fillWinding);
+	apc_gp_set_fill_mode( self, sys fillMode);
 	apc_gp_set_fill_pattern_offset( self, sys fillPatternOffset);
 	apc_gp_set_line_width( self, sys lineWidth);
 	apc_gp_set_line_end( self, sys lineEnd);
@@ -1516,12 +1516,13 @@ hwnd_enter_paint( Handle self)
 	apc_gp_set_line_pattern( self,
 		( Byte*)(( sys linePatternLen > 3) ? sys linePattern : ( Byte*)&sys linePattern),
 		sys linePatternLen);
+	apc_gp_set_miter_limit( self, sys miterLimit);
 	apc_gp_set_rop( self, sys rop);
 	apc_gp_set_rop2( self, sys rop2);
 	apc_gp_set_transform( self, sys transform. x, sys transform. y);
 	apc_gp_set_fill_pattern( self, sys fillPattern2);
 	sys psd-> font           = var font;
-	sys psd-> fillWinding    = sys fillWinding;
+	sys psd-> fillMode       = sys fillMode;
 	sys psd-> fillPatternOffset = sys fillPatternOffset;
 	sys psd-> lineWidth      = sys lineWidth;
 	sys psd-> lineEnd        = sys lineEnd;
@@ -1560,7 +1561,7 @@ hwnd_leave_paint( Handle self)
 	}
 	if ( sys psd != nil) {
 		var font           = sys psd-> font;
-		sys fillWinding    = sys psd-> fillWinding;
+		sys fillMode       = sys psd-> fillMode;
 		sys fillPatternOffset  = sys psd-> fillPatternOffset;
 		sys lineWidth      = sys psd-> lineWidth;
 		sys lineEnd        = sys psd-> lineEnd;
